﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;                 //【Bitmap】【Color】
using System.Drawing.Imaging;         //【BitmapData】
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;             //【Debug】
using System.Runtime.InteropServices; //【Marshal.Copy】
using Microsoft.Kinect;               //【参照設定】プロジェクト(P)⇒参照の追加(F)⇒.NETタブ⇒Microsoft.Kinectを選択

namespace Bitmap_GetColorImage
{
    public partial class Form1 : Form
    {
        //【KINECT関連】--------------------------------------------------------------------
        private KinectSensor KS_KinectSensor;   //【KINECT】
        private ColorImageFrame KS_ColorFrame;  //【KINECTのColorImageFrame】
        private byte[] KS_ColorImageDataXRGB;   //【KINECTのColorImageData】32bit/画素,X8R8G8B8表示形式
        private int KS_ColorFrameWidth;         //【KINECTのColorFrameの横方向画素数】
        private int KS_ColorFrameHeight;        //【KINECTのColorFrameの縦方向画素数】
        private Bitmap KS_bmp1;                 //【ColorImage表示用Bitmapオブジェクト】
        private BitmapData KS_data1;            //【ColorImage用BitmapData】
        private Rectangle KS_rect1;             //【ColorImage表示枠】
        private bool KS_flagFlip = true;        //【左右反転フラグ】

        public Form1()
        {
            InitializeComponent();
            //【利用可能なKINECTセンサを取得する】
            KS_KinectSensor = (from sensorToCheck in KinectSensor.KinectSensors where sensorToCheck.Status == KinectStatus.Connected select sensorToCheck).FirstOrDefault();
            if (KS_KinectSensor == null)
            {
                pictureBox2.Image = imageList1.Images[1];
                MessageBox.Show(
                   "接続可能なKINECTセンサが見当たりません。",
                   "KINECT for Windows Error Message",
                   MessageBoxButtons.OK
                );
                return;
            }
            else
            {
                pictureBox2.Image = imageList1.Images[0];
            }

            //【KINECTの設定】ColorStreamを許可*******************************************************************
            KS_KinectSensor.ColorStream.Enable(ColorImageFormat.RgbResolution640x480Fps30); //【ColorStream許可】
            //■他に.RgbResolution1280x960Fps12も有り。ColorImageFormatを参照の事。

            //【KINECT動作開始】**********************************************************************************
            KS_KinectSensor.Start(); //【KINECTスタート】

            //【KINECTデータ取得イベントの設定とデータバッファの確保】********************************************
            KS_KinectSensor.ColorFrameReady += ColorImageReady; //【準備完了でColorImageReadyがcallされる】
            KS_ColorImageDataXRGB = new byte[KS_KinectSensor.ColorStream.FramePixelDataLength];      //【カラー画像データ用】
            KS_ColorFrameWidth = KS_KinectSensor.ColorStream.FrameWidth;   //【ColorImageの横方向画素数】
            KS_ColorFrameHeight = KS_KinectSensor.ColorStream.FrameHeight; //【ColorImageの縦方向画素数】
            KS_KinectSensor.ElevationAngle = 0; //【KINECTセンサの上下角を0°に設定】設定範囲±27°
            KS_bmp1 = new Bitmap(KS_ColorFrameWidth, KS_ColorFrameHeight, System.Drawing.Imaging.PixelFormat.Format32bppRgb); //【ColorImage表示用】
            KS_rect1 = new Rectangle(0, 0, KS_ColorFrameWidth, KS_ColorFrameHeight); //【ColorImage用枠】

        }

        //***************************************************************
        //【ColorImageの取得イベント】
        //***************************************************************
        private void ColorImageReady(object sender, ColorImageFrameReadyEventArgs e)
        {
            using (KS_ColorFrame = e.OpenColorImageFrame())
            {
                if (KS_ColorFrame != null)
                {
                    KS_ColorFrame.CopyPixelDataTo(KS_ColorImageDataXRGB); //【KINECTのColor画像を配列PixelDataにコピー】
                    KS_data1 = KS_bmp1.LockBits(KS_rect1, ImageLockMode.WriteOnly, System.Drawing.Imaging.PixelFormat.Format32bppRgb); //【Lock】
                    Marshal.Copy(KS_ColorImageDataXRGB, 0, KS_data1.Scan0, KS_ColorImageDataXRGB.Length); //【UnmanagedMemoryのBitmapDataに配列PixelDataをコピー】
                    KS_bmp1.UnlockBits(KS_data1);           //【Lock解除】
                    if (KS_flagFlip) { KS_bmp1.RotateFlip(RotateFlipType.Rotate180FlipY); } //【左右反転】
                    pictureBox1.Image = KS_bmp1;            //【pictureBox1に画像表示】
                    //             Application.DoEvents();
                }
                else
                {
                    //【ImageFrameがnullだったのは要求が間に合わなかったと考えられます】
                }
            }
        }

        //***************************************************************
        //【終了要求ボタン】
        //***************************************************************
        private void button1_Click(object sender, EventArgs e)
        {
            if (KS_KinectSensor != null)
            {
                KS_KinectSensor.Stop();    //【KINECTセンサを終了】★これを実行しないとレーザーが消えない
                KS_KinectSensor.Dispose(); //【KINECTセンサのオブジェクトを解放する】
            }
            this.Close();              //【Formを閉じる】
        }

        //***************************************************************
        //【上下角変更ボタン】KINECTの上下角を変更。±27°の範囲内。時間を置かないとエラーが発生する
        //***************************************************************
        private void button2_Click(object sender, EventArgs e)
        {
            Int32 angle = int.Parse(textBox1.Text); //【設定角度を読み込む】±27°
            if (angle < KS_KinectSensor.MaxElevationAngle && angle > KS_KinectSensor.MinElevationAngle)
            {
                KS_KinectSensor.ElevationAngle = angle; //【上下角の変更】
            }
        }

        //***************************************************************
        //【カラー画像の左右反転指定フラグ】
        //***************************************************************
        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox1.Checked) { KS_flagFlip = true; } else { KS_flagFlip = false; }
        }
    }
}
